/* START LIBRARY DESCRIPTION **************************
FILE:				DDS Comb Commands.c
PARENT FILE:	DDS Comb.c
DESCRIPTION:	Command parsing and execution functions.
END DESCRIPTION **************************************/


/********* Function PROTOTYPES (follow syntax, including comment lines) ***************************/
/*** BeginHeader CommandHandler, etc */
void CommandHandler (char* Cmd);
int FreqSet (char* Cmd);
int AmplitudeSet (char* Cmd);
int PhaseSet (char* Cmd);
int SweepSet (char* Cmd);
void SyncPhase (void);
int RampSet (char* Cmd);
/*** EndHeader ***********************/

#ifndef DO_DEBUG
	#nodebug								// disables insertion of breakpoints for debugging, this file only.
#endif


/*************************** Incoming Command From Host *******************************************/
/* START FUNCTION DESCRIPTION ******************************************************
CommandHandler    <DDS Comb Commands.c>
DESCRIPTION:	Command(s) arrived from host.  Parse out each command and process it.
NOTES:			1) The null terminated command string is the global var Cmd.
					2) Invalid commands result in removal of the whole command string.
END DESCRIPTION *******************************************************************/
void CommandHandler (char* Cmd)  {
	if (HostSock.hisaddr == 0)  Cmd[0] = 0x00;							//if no host, ignore all commands.

	while (strlen(Cmd))  {
		switch (Cmd[0])  {
			case 'H':																	 //INCOMING HEARTBEAT (1 byte).
				udp_send(&HostSock, Cmd, 1);													 //echo the heartbeat.
				Cmd += 1;																				  //remove command.
  				break;

			case 'V':																		  //RABBIT VERSION (1 byte).
				sprintf (Buf, "V%s", VERSION);
				udp_send(&HostSock, Buf, strlen(Buf));
				Cmd += 1;																				  //remove command.
  				break;

			case 'F':																	//SET A FREQUENCY (7-13 bytes).
				if (strlen(Cmd) >= 7)  Cmd += FreqSet (Cmd);										//valid command.
				else  Cmd[0] = 0x00;																	 //invalid command.
				break;

			case 'A':																	//SET AN AMPLITUDE (4-7 bytes).
				if (strlen(Cmd) >= 5)  Cmd += AmplitudeSet (Cmd);								//valid command.
				else  Cmd[0] = 0x00;																	 //invalid command.
				break;

			case 'P':														              //SET A PHASE (4-7 bytes).
				if (strlen(Cmd) >= 5)  Cmd += PhaseSet (Cmd);	     							//valid command.
				else  Cmd[0] = 0x00;																	 //invalid command.
				break;

			case 'S':																//SWEEP A FREQUENCY (25-39 bytes).
				if (strlen(Cmd) >= 25)  Cmd += SweepSet (Cmd);	 								//valid command.
				else  Cmd[0] = 0x00;																	 //invalid command.
				break;

			case 'U':												  //SET AMPLITUDE RAMPING UP/DOWN (5-7 bytes).
				if (strlen(Cmd) >= 5)  Cmd += RampSet (Cmd);										//valid command.
				else  Cmd[0] = 0x00;																	 //invalid command.
				break;

			case 'R':														//RESET PHASES OF ALL CHANNELS (1 byte).
				SyncPhase();
				Cmd += 1;																				  //remove command.
  				break;

			default:																						 //NO SUCH COMMAND.
				Cmd[0] = 0x00;
				break;
		}
	}
}


/************************************ Set A Frequency *********************************************/
/* START FUNCTION DESCRIPTION *****************************************
FreqSet    <DDS Comb Commands.c>
DESCRIPTION:	Set the frequency of a channel.
PARAM 1:			The command string, starting with the frequency command.
RETURNS:			Length of this command.
END DESCRIPTION ******************************************************/
int FreqSet (char* Cmd)  {
	int Ch, Ptr, Spc;
	long Freq;
	char Werd[5];

	Ch = (int)Cmd[1] - 65;																  //hold onto channel (0-3).
	Cmd += 3;																								//remove header.
	Freq = atol(Cmd);																				  //extract frequency.
	Spc = (int)(strstr(Cmd, " ") - Cmd);													//find trailing space.


	if (Spc > 2  &&  Spc < 12									//has trailing space, good freq, good channel.
	 && Freq >= F_MIN  &&  Freq <= F_MAX
	 && Ch >= 0  &&  Ch <= 3)  {

		Werd[0] = CSR;																		  //Channel Select Register.
		Werd[1] = 0x10 << Ch;
		DdsWrite (Werd);

		Werd[0] = CFR;																		//Channel Function Register.
		Werd[1] = 0x00;																			//modulation disabled.
		Werd[2] = Bit0 + Bit1 + Bit5;			  //DAC high, IO_UPDATE loads SRRs, linear sweep disabled.
		Werd[3] = 0x00;
		DdsWrite (Werd);
		CfrShadow[1][Ch] = Werd[1];
		CfrShadow[2][Ch] = Werd[2];
		CfrShadow[3][Ch] = Werd[3];

		Werd[0] = CFTW0;															//Channel Frequency Tuning Word 0.
		Freq = (long)((float)Freq * (float)FREQ2FTW);				  //convert to Frequency Tuning Word.
		for (Ptr = 1; Ptr < 5;  Ptr++)							  //prepare for register loading, MS to LS.
			Werd[Ptr] = *(((char*)&Freq) + 4 - Ptr);		//NB: verify little or big endian if editing!!
		DdsWrite (Werd);

		ImplementBuffers();
	}
	else  Spc = -4;																					 //invalid command.

	return (Spc + 4);																//return the size of this command.
}


/************************************ Set An Ampltitude *******************************************/
/* START FUNCTION DESCRIPTION *****************************************
AmplitudeSet    <DDS Comb Commands.c>
DESCRIPTION:	Set the amplitude of a channel.
PARAM 1:			The command string, starting with the amplitude command.
RETURNS:			Length of this command.
END DESCRIPTION ******************************************************/
int AmplitudeSet (char* Cmd)  {
	int Ch, Ptr, Spc, Ampl;
	char Werd[4];

	Ch = (int)Cmd[1] - 65;																  //hold onto channel (0-3).
	Cmd += 3;																								//remove header.
	Ampl = (int)atol(Cmd);																		  //extract amplitude.

	Spc = (int)(strstr(Cmd, " ") - Cmd);													//find trailing space.
	if (Spc > 0  &&  Spc < 6							 //has trailing space, good amplitude, good channel.
	 && Ampl >= 0  &&  Ampl <= 100
	 && Ch >= 0  &&  Ch <= 3)  {

		Amplitude[Ch] = (int)((float)Ampl * 10.23);							 //where 10.23 is 2^10-1 / 100.

		Werd[0] = CSR;																		  //CHANNEL SELECT REGISTER.
		Werd[1] = 0x10 << Ch;
		DdsWrite (Werd);

		Werd[0] = ACR;																	  //AMPLITUDE CONTROL REGISTER.
		Werd[1] = (char)RampUD[Ch];
		Werd[2] = (char)(Amplitude[Ch] / 256 + Bit4);		 //MS bits and amplitude multiplier enable.
		if (RampUD[Ch])  Werd[2] += Bit3 + Bit6 + Bit7;										  //enable ramping.
		Werd[3] = (char)(Amplitude[Ch] % 256);
		DdsWrite (Werd);

		ImplementBuffers();
	}
	else  Spc = -4;																					 //invalid command.

	return (Spc + 4);																//return the size of this command.
}


/************************************ Set A Phase *************************************************/
/* START FUNCTION DESCRIPTION *************************************
PhaseSet    <DDS Comb Commands.c>
DESCRIPTION:	Set the phase of a channel.
PARAM 1:			The command string, starting with the phase command.
RETURNS:			Length of this command.
END DESCRIPTION **************************************************/
int PhaseSet (char* Cmd)  {
	int Ch, Ptr, Spc;
	long Phi;
	char Werd[4];

	Ch = (int)Cmd[1] - 65;																  //hold onto channel (0-3).
	Cmd += 3;																								//remove header.
	Phi = atol(Cmd);																						//extract phase.
	Spc = (int)(strstr(Cmd, " ") - Cmd);													//find trailing space.
	if (Spc > 0  &&  Spc < 6							 //has trailing space, good amplitude, good channel.
	 && Phi >= 0  &&  Phi <= 359
	 && Ch >= 0  &&  Ch <= 3)  {

		Werd[0] = CSR;																		  //Channel Select Register.
		Werd[1] = 0x10 << Ch;
		DdsWrite (Werd);

		Werd[0] = CPOW0;																 //CHANNEL PHASE OFFSET WORD 0.
		Phi = (long)((float)Phi * 45.508);										//where 45.508 is 2^14-1 / 360.
		Werd[1] = (char)(Phi / 256);
		Werd[2] = (char)(Phi % 256);
		DdsWrite (Werd);

		ImplementBuffers();
	}
	else  Spc = -4;																					 //invalid command.

	return (Spc + 4);																//return the size of this command.
}


/************************************ Set A Sweep *************************************************/
/* START FUNCTION DESCRIPTION *************************************
SweepSet    <DDS Comb Commands.c>
DESCRIPTION:	Sweep a channel.
PARAM 1:			The command string, starting with the sweep command.
RETURNS:			Length of this command.
END DESCRIPTION **************************************************/
int SweepSet (char* Cmd)  {
	int Ch, Ptr, Spc, CmdLen, GoodCmd;
	long Ftw, FreqHigh, FreqLow, StepSize, StepTime;
	char Werd[5];

	Ch = (int)Cmd[1] - 65;																			 //EXTRACT CHANNEL.
	GoodCmd = 1;										  //assume that the command is good, until shown wrong.
	if (Ch < 0  ||  Ch > 3) GoodCmd = 0;
	Cmd += 3;																				//remove header and channel.
	CmdLen = 3;																 //this command is already 3 bytes long.

	FreqHigh = atol(Cmd);																	//EXTRACT HIGH FREQUENCY.
	if (FreqHigh < F_MIN  ||  FreqHigh > F_MAX)  GoodCmd = 0;					//frequency out of range.
	Spc = (int)(strstr(Cmd, " ") - Cmd);													//find trailing space.
	if (Spc < 1  ||  Spc > 9)  GoodCmd = 0;														//missing space.
	Cmd = strstr(Cmd, " ") + 1;  															 //remove high frequency.
	CmdLen += Spc + 1;

	FreqLow = atol(Cmd);																		 //EXTRACT LOW FREQUENCY.
	Spc = (int)(strstr(Cmd, " ") - Cmd);													//find trailing space.
	if (Spc < 1  ||  Spc > 9)  GoodCmd = 0;														//missing space.
	if (FreqLow < F_MIN  ||  FreqLow > F_MAX)  GoodCmd = 0;						//frequency out of range.
	if (FreqLow > FreqHigh)  GoodCmd = 0;													//frequencies overlap.
	Cmd = strstr(Cmd, " ") + 1;															  //remove low frequency.
	CmdLen += Spc + 1;

	StepSize = atol(Cmd);																		  //EXTRACT STEP SIZE.
	Spc = (int)(strstr(Cmd, " ") - Cmd);													//find trailing space.
	if (Spc < 1  ||  Spc > 9)  GoodCmd = 0;														//missing space.
	if (StepSize <= 0  ||  StepSize > F_MAX)  GoodCmd = 0;						//step size out of range.
	Cmd = strstr(Cmd, " ") + 1;																	//remove step size.
	CmdLen += Spc + 1;

	StepTime = atol(Cmd);																		  //EXTRACT STEP TIME.
	Spc = (int)(strstr(Cmd, " ") - Cmd);													//find trailing space.
	if (Spc < 1  ||  Spc > 5)  GoodCmd = 0;														//missing space.
	if (StepTime < 8  ||  StepTime > 2048)  GoodCmd = 0;							//step time out of range.
	StepTime >>= 3;																					//reduce to 8-bits.
	CmdLen += Spc + 1;

	if (GoodCmd)  {
		Werd[0] = CSR;																		  //Channel Select Register.
		Werd[1] = 0x10 << Ch;
		DdsWrite (Werd);

		Werd[0] = CFR;																		//Channel Function Register.
		Werd[1] = Bit7; //(MS)																  //frequency sweep mode.
		Werd[2] = Bit0 + Bit1 + Bit5 + Bit6;	//DAC high, IO_UPDATE loads SRRs, linear sweep enabled.
		Werd[3] = 0x00;
		DdsWrite (Werd);
		CfrShadow[1][Ch] = Werd[1];
		CfrShadow[2][Ch] = Werd[2];
		CfrShadow[3][Ch] = Werd[3];

		if (Ch > 0)  {                                                   //AMPLITUDE CONTROL REGISTER.
	      Werd[0] = ACR;																			//no ramping on CH1-3.
	      Werd[1] = 0x00; //(MS)
	      Werd[2] = (char)(Amplitude[Ch] / 256 + Bit4);    //MS bits and amplitude multiplier enable.
	      Werd[3] = (char)(Amplitude[Ch] % 256);
	      DdsWrite (Werd);
		}

		Werd[0] = CFTW0;								//Channel Frequency Tuning Word 0 (start, or low freq).
		Ftw = (long)((float)FreqLow * (float)FREQ2FTW);				  //convert to Frequency Tuning Word.
		for (Ptr = 1; Ptr < 5;  Ptr++)							  //prepare for register loading, MS to LS.
			Werd[Ptr] = *(((char*)&Ftw) + 4 - Ptr);		//NB: verify little or big endian if editing!!
		DdsWrite (Werd);

		Werd[0] = CW1;															//Channel Word 1 (end, or high freq).
		Ftw = (long)((float)FreqHigh * (float)FREQ2FTW);			  //convert to Frequency Tuning Word.
		for (Ptr = 1; Ptr < 5;  Ptr++)							  //prepare for register loading, MS to LS.
			Werd[Ptr] = *(((char*)&Ftw) + 4 - Ptr);		//NB: verify little or big endian if editing!!
		DdsWrite (Werd);

		Werd[0] = RDW;																				  //Rising Delta Word.
		Ftw = (long)((float)StepSize * (float)FREQ2FTW);			  //convert to Frequency Tuning Word.
		for (Ptr = 1; Ptr < 5;  Ptr++)							  //prepare for register loading, MS to LS.
			Werd[Ptr] = *(((char*)&Ftw) + 4 - Ptr);		//NB: verify little or big endian if editing!!
		DdsWrite (Werd);

		Werd[0] = FDW;																				 //Falling Delta Word.
		Ftw = (long)((float)StepSize * (float)FREQ2FTW);			  //convert to Frequency Tuning Word.
		for (Ptr = 1; Ptr < 5;  Ptr++)							  //prepare for register loading, MS to LS.
			Werd[Ptr] = *(((char*)&Ftw) + 4 - Ptr);		//NB: verify little or big endian if editing!!
		DdsWrite (Werd);

		Werd[0] = LSRR;																		//Linear Sweep Ramp Rate.
		Werd[1] = (char)StepTime; //(MS)														 //FSRR, falling rate.
		Werd[2] = (char)StepTime;		  														 //RSRR, rising rate.
		DdsWrite (Werd);

		ImplementBuffers();
	}
	else  CmdLen = 0;																					 //invalid command.

	return (CmdLen);
}


/********************************* Set An Amplitude Ramp Rate *************************************/
/* START FUNCTION DESCRIPTION *****************************************
RampSet    <DDS Comb Commands.c>
DESCRIPTION:	Set the amplitude ramp rate up/down of a channel.
PARAM 1:			The command string, starting with the ramp rate command.
RETURNS:			Length of this command.
NOTES:         Rate of 0 means no ramping, valid range is 1-255uS.
END DESCRIPTION ******************************************************/
int RampSet (char* Cmd)  {
	int Ch, Ptr, Spc, Ramp;
	char Werd[4];

	Ch = (int)Cmd[1] - 65;																  //hold onto channel (0-3).
	Cmd += 3;																								//remove header.
	Ramp = (int)atol(Cmd);																		  //extract ramp rate.
	Spc = (int)(strstr(Cmd, " ") - Cmd);													//find trailing space.
	if (Spc > 0  &&  Spc < 6																			//Valid Command.
	 && Ramp >= 0  &&  Ramp <= 255
	 && Ch >= 0  &&  Ch <= 3)  {

		RampUD[Ch] = Ramp;

		Werd[0] = CSR;																		  //CHANNEL SELECT REGISTER.
		Werd[1] = 0x10 << Ch;
		DdsWrite (Werd);

		Werd[0] = ACR;																	  //AMPLITUDE CONTROL REGISTER.
		Werd[1] = (char)RampUD[Ch];
		Werd[2] = (char)(Amplitude[Ch] / 256 + Bit4);		 //MS bits and amplitude multiplier enable.
		if (RampUD[Ch])  Werd[2] += Bit3 + Bit6 + Bit7;										  //enable ramping.
		Werd[3] = (char)(Amplitude[Ch] % 256);
		DdsWrite (Werd);

		ImplementBuffers();
	}
	else  Spc = -4;																					 //invalid command.

	return (Spc + 4);																//return the size of this command.
}


/************************************ Sync All Phases *********************************************/
/* START FUNCTION DESCRIPTION *********************
SyncPhase    <DDS Comb Commands.c>
DESCRIPTION:	Reset channels' absolute phases now.
END DESCRIPTION **********************************/
void SyncPhase (void)  {
	int Ch;
	char Werd[4];

	for (Ch = 0;  Ch < 4;  Ch++)  {
      Werd[0] = CSR;																		  //Channel Select Register.
      Werd[1] = 0x10 << Ch;
      DdsWrite (Werd);

      Werd[0] = CFR;																		//Channel Function Register.
      Werd[1] = CfrShadow[1][Ch];
      Werd[2] = CfrShadow[2][Ch];
      Werd[3] = CfrShadow[3][Ch]  |  Bit2;					//autoclear phase accumulator on IO_UPDATE.
      DdsWrite (Werd);
	}
	ImplementBuffers();

	for (Ch = 0;  Ch < 4;  Ch++)  {
      Werd[0] = CSR;																		  //Channel Select Register.
      Werd[1] = 0x10 << Ch;
      DdsWrite (Werd);

      Werd[0] = CFR;																		//Channel Function Register.
      Werd[1] = CfrShadow[1][Ch];
      Werd[2] = CfrShadow[2][Ch];
      Werd[3] = CfrShadow[3][Ch];
      DdsWrite (Werd);
	}
	ImplementBuffers();
}

/**************************************************************************************************/


